// ============================================================================
// Module Name: IPat_SyncVar
// Revision: 20170923a
// Author: Altmo toolbox
// Description:
//  + Double FF synchronizer with variable width
// ============================================================================

`timescale 1ps/100fs

module IPat_SyncVar (
    clk,
    i_sync,
    o_sync,
    rst_n
);

// parameter ==================================================================
parameter P_IPAT_SYNCVAR_DWIDTH = 8;	// width of q
parameter P_IPAT_SYNCVAR_PDEPTH = 2;	// depth of pipe

// local parameter ============================================================
localparam LP_DWIDTH = P_IPAT_SYNCVAR_DWIDTH;
localparam LP_PDEPTH = P_IPAT_SYNCVAR_PDEPTH;

localparam LP_UD = 1;	// unit delay to avoid racing

// port definition ============================================================
input			clk;	// clock
input	[LP_DWIDTH-1:0]	i_sync;	// data input
output	[LP_DWIDTH-1:0]	o_sync;	// data output
input			rst_n;	// reset 0 active


// register and wire definition ===============================================
reg	[LP_DWIDTH-1:0]	r_sync[0:LP_PDEPTH-1];

// logic ======================================================================

genvar i;
generate
    for (i=0; i<LP_PDEPTH; i=i+1) begin: syncvar
        always @(posedge clk or negedge rst_n) begin
            if (rst_n == 1'b0) begin
                r_sync[i] <= #LP_UD {LP_DWIDTH{1'b0}};
            end
            else begin
                if (i==0) begin
                    r_sync[i] <= #LP_UD i_sync;
                end
                else begin
                    r_sync[i] <= #LP_UD r_sync[i-1];
                end
            end
        end
    end
endgenerate

assign o_sync = r_sync[LP_PDEPTH-1];

endmodule
